<?php

/**
 * Redaxo Addon
 * Ein Article Slice kann veschoben oder kopiert werden
 *
 * @author Michael Ziem
 * @version 0.9.3
 */

/**
 * Zentrale Klasse mit allen Methoden
 *
 * @author Michael Ziem
 */
class rex_sliceshift {

    /**
     * Fügt das benötige Stylesheet ins Backend ein
     *
     * @param array $params Extension-Point Parameter
     */
    public static function addCSS($params) {

        global $REX;

        $media_dir = isset($REX['MEDIA_ADDON_DIR']) ? $REX['MEDIA_ADDON_DIR'] : 'files/addons';

        $params['subject'] .= PHP_EOL . '<link rel="stylesheet" type="text/css" href="../' . $media_dir . '/sliceshift/backend.css" />';

        return $params['subject'];
    }

    /**
     * Fügt einen Button oder das Auswahlmenu ein
     *
     * @param array $params Extension-Point Parameter
     */
    public static function addSliceShift($params) {

        global $REX, $I18N;

        $category_id = rex_request('category_id', 'rex-category-id', '');
        $slice_id = rex_request('slice_id', 'rex-slice-id', '');
        $function = rex_request('function', 'string');

        if($function == 'sliceshift' && $slice_id == $params['slice_id']) {

            // Formular
            $add = '<form action="" class="sliceshift" method="get">';

            $add .= '<input type="hidden" name="page" value="content">';
            $add .= '<input type="hidden" name="slice_id" value="' . $params['slice_id'] . '">';
            $add .= '<input type="hidden" name="clang" value="' . $params['clang'] . '">';
            $add .= '<input type="hidden" name="ctype" value="' . $params['ctype'] . '">';
            $add .= '<input type="hidden" name="article_id" value="' . $params['article_id'] . '">';
            $add .= '<input type="hidden" name="module_id" value="' . $params['module_id'] . '">';

            // Button "abbrechen"
            $add .= '<a href="index.php?page=content&amp;mode=edit&amp;clang=' . $params['clang'] . '&amp;ctype=' . $params['ctype'] . '&amp;article_id=' . $params['article_id'] . '&amp;category_id=' . $category_id . '" class="right" title="' . $I18N->msg('sliceshift_cancel') . '">' . $I18N->msg('sliceshift_cancel_x') . '</a>';

            // Radio Button "verschieben"
            $add .= '<div class="rex-form-row">';
            $add .= '<p class="rex-form-radio">';
            $add .= '<input class="rex-form-radio" type="radio" id="sliceshift_move" name="function" value="sliceshift_move" checked="checked">';
            $add .= '<label for="sliceshift_move">' . $I18N->msg('sliceshift_move') . '</label>';
            $add .= '</p>';
            $add .= '</div>';

            // Radio Button "kopieren"
            $add .= '<div class="rex-form-row">';
            $add .= '<p class="rex-form-radio">';
            $add .= '<input class="rex-form-radio" type="radio" id="sliceshift_copy" name="function" value="sliceshift_copy">';
            $add .= '<label for="sliceshift_copy">' . $I18N->msg('sliceshift_copy') . '</label>';
            $add .= '</p>';
            $add .= '</div>';

            // Select Artikel
            $add .= '<div class="rex-form-row">';
            $add .= self::getArticlesSelect($params);
            $add .= '</div>';

            // Select Spalten wenn mehr als eine vorhanden...
            if(count($REX['CTYPE']) > 0) {
                $ctype_select = new rex_select();
                $ctype_select->setStyle('class="rex-form-select"');
                $ctype_select->setName('sliceshift_ctype');
                $ctype_select->setSize('1');
                $ctype_select->setSelected($params['ctype']);
                // Kompatibilität
                if(method_exists($ctype_select, 'addOptgroup')) {
                    $ctype_select->addOptgroup($I18N->msg('sliceshift_ctype'));
                }
                $ctype_select->addArrayOptions($REX['CTYPE']);

                $add .= '<div class="rex-form-row">';
                $add .= $ctype_select->get();
                $add .= '</div>';
            // ...ansonsten Slice ID = 1
            } else {
                $add .= '<input type="hidden" name="sliceshift_ctype" value="1">';
            }

            // Select Sprachen wenn mehr als eine vorhanden...
            if(count($REX['CLANG']) > 1) {
                $clang_select = new rex_select();
                $clang_select->setStyle('class="rex-form-select"');
                $clang_select->setName('sliceshift_clang');
                $clang_select->setSize('1');
                $clang_select->setSelected($params['clang']);
                // Kompatibilität
                if(method_exists($clang_select, 'addOptgroup')) {
                    $clang_select->addOptgroup($I18N->msg('sliceshift_lang'));
                }
                $clang_select->addArrayOptions($REX['CLANG']);

                $add .= '<div class="rex-form-row">';
                $add .= $clang_select->get();
                $add .= '</div>';
            // ...ansonsten Sprachen ID = 0
            } else {
                $add .= '<input type="hidden" name="sliceshift_clang" value="0">';
            }

            // Button "Ausführen"
            $add .= '<input class="submit" type="submit" value="' . $I18N->msg('sliceshift_execute') . '" title="' . $I18N->msg('sliceshift_execute') . '">';

            $add .= '</form>';

            // ...in vorhandenens Array einfügen
            array_splice($params['subject'], 1, 0, $add);

        } else {

            // Button...
            $add = '<a href="index.php?page=content&amp;article_id=' . $params['article_id'] . '&amp;category_id=' . $category_id . '&amp;mode=edit&amp;slice_id=' . $params['slice_id'] . '&amp;clang=' . $params['clang']. '&amp;ctype=' . $params['ctype']. '&amp;function=sliceshift#slice' . $params['slice_id'] . '" title="Verschieben oder kopieren" class="rex-tx3">verschieben</a>';

            // ...in vorhandenens Array einfügen
            array_splice($params['subject'], 1, 0, $add);
        }

        return $params['subject'];
    }

    /**
     * Slice bewegen aufrufen
     *
     * @param array $params Extension-Point Parameter
     */
    static function doSliceShift($params) {
        try {
            self::sliceShift($params);
        } catch(Exception $e) {
            echo rex_warning($e->getMessage());
            return false;
        }
    }

    /**
     * Slice bewegen
     *
     * @param array $params Extension-Point Parameter
     */
    static function sliceShift($params) {

        global $REX, $I18N;

        $function = rex_request('function', 'string');

        if($function == 'sliceshift_copy' || $function == 'sliceshift_move' ) {

            $article_id  = rex_request('article_id', 'rex-article-id');
            $clang = rex_request('clang', 'rex-clang-id', $REX['START_CLANG_ID']);
            $ctype = rex_request('ctype', 'rex-ctype-id', 1);
            $slice_id = rex_request('slice_id', 'rex-slice-id', '');
            $module_id = rex_request('module_id', 'rex-module-id', '');

            $sliceshift_article_id = rex_request('sliceshift_article_id', 'rex-slice-id', '');
            $sliceshift_ctype = rex_request('sliceshift_ctype', 'int');
            $sliceshift_clang = rex_request('sliceshift_clang', 'int');
            
            // Default value
            $sliceshift_revision = 0;

            // Die Revision ID des "version" Addons ist in der Session gespeichert
            // See: /redaxo/include/addons/version/config.inc.php Zeile 66
            
            // Leider wird die SESSION variable bei deinstallieren des "version" Addons nicht gelöscht
            // deshalb überprüfen wir, ob das "version" Addons aktiv ist
            if($REX['ADDON']['status']['version'] == 1) {
                $rex_version_article = $REX['LOGIN']->getSessionVar('rex_version_article');

                // Wenn das "version" Addon nicht aktiv ist, dann wird revision = 0 gesetzt
                if (isset($rex_version_article) && is_array($rex_version_article) && is_integer($rex_version_article[$article_id])) {
                    $sliceshift_revision = $rex_version_article[$article_id];
                }
            }

            // Artikel holen
            $article_to = rex_sql::factory();
            $article_to->setQuery('
            SELECT article.*, template.attributes as template_attributes
            FROM ' . $REX['TABLE_PREFIX'] . 'article as article
            LEFT JOIN ' . $REX['TABLE_PREFIX'] . 'template as template ON template.id = article.template_id
            WHERE article.id = ' . $sliceshift_article_id . '
                AND clang = ' . $sliceshift_clang);

            if ($article_to->getRows() == 1) {

                // Attribute auslesen
                $template_attributes = $article_to->getValue('template_attributes');

                // Überprüfen, ob der Modultyp im Ziel-Artikel erlaubt ist
                if(!rex_template::hasModule($template_attributes, $sliceshift_ctype, $module_id)) {
                     // Wenn Fehler, dann ausgeben
                    throw new Exception($I18N->msg('sliceshift_msg_error_slice_notallowed'));
                }

                // Überprüfen, ob die Spalte im Zielartikel existiert
                if(!self::hasCtype($template_attributes, $sliceshift_ctype)) {
                    // Wenn Fehler, dann ausgeben
                    throw new Exception($I18N->msg('sliceshift_msg_error_noctype'));
                }
            }

            try {
                $copy_article = self::copySlice(
                    // Id des Quell-Artikels
                    $article_id,
                    // Id des Ziel-Artikels
                    $sliceshift_article_id,
                    // Spalte Ziel-Artikels
                    $sliceshift_ctype,
                    // Slice ID
                    $slice_id,
                    // ID der Zielsprache
                    $sliceshift_clang,
                    // Revision des Zielartikels
                    $sliceshift_revision
                );
            } catch(Exception $e) {
                echo rex_warning($e->getMessage());
                return false;
            }

            $message = $I18N->msg('slice_moved');

            if($copy_article) {

                // Slice löschen
                if($function == 'sliceshift_move') {
                    rex_deleteSlice($slice_id);

                    // ----- EXTENSION POINT
                    $message = rex_register_extension_point('SLICE_DELETED', $message,
                        array(
                            'article_id' => $sliceshift_article_id,
                            'clang' => $sliceshift_clang,
                            'function' => $function,
                            'mode' => $params['mode'],
                            'slice_id' => $slice_id,
                            'page' => 'content',
                            'ctype' => $sliceshift_ctype,
                            'category_id' => $params['category_id'],
                            'module_id' => $module_id,
                            'article_revision' => $params['article_revision'],
                            'slice_revision' => $params['slice_revision']
                        )
                    );

                }

                // Cache des Quell-Artikels löschen
                rex_deleteCacheArticleContent($article_id, $clang);

                 // Cache des Ziel-Artikels löschen
                rex_deleteCacheArticleContent($sliceshift_article_id, $clang);

                // ----- EXTENSION POINT
                $message = rex_register_extension_point('SLICE_SHIFT_SHIFTED', $message,
                    array(
                        'article_id' => $article_id,
                        'to_article_id' => $sliceshift_article_id,
                        'clang' => $clang,
                        'to_clang' => $sliceshift_clang,
                        'ctype' => $ctype,
                        'to_ctype' => $sliceshift_ctype,
                        'function' => $function,
                        'slice_id' => $slice_id,
                        'page' => 'content',
                    )
                );

                // ----- EXTENSION POINT
                $message = rex_register_extension_point('SLICE_ADDED', $message,
                    array(
                        'article_id' => $sliceshift_article_id,
                        'clang' => $sliceshift_clang,
                        'function' => $function,
                        'mode' => $params['mode'],
                        'slice_id' => $slice_id,
                        'page' => 'content',
                        'ctype' => $sliceshift_ctype,
                        'category_id' => $params['category_id'],
                        'module_id' => $module_id,
                        'article_revision' => $params['article_revision'],
                        'slice_revision' => $params['slice_revision']
                    )
                );
            }
        }
    }

    /**
     * HTML Select
     *
     * @param array $params 	Extension-Point Parameter
     *
     * @return string HTML-Element
     */
    public static function getArticlesSelect($params) {

        global $I18N;

        $article_select = new rex_select();
        $article_select->setStyle('class="rex-form-select"');
        $article_select->setName('sliceshift_article_id');
        $article_select->setSize('1');
        $article_select->setSelected($params['article_id']);
        // Kompatibilität
        if(method_exists($article_select, 'addOptgroup')) {
            $article_select->addOptgroup($I18N->msg('sliceshift_article'));
        }

        // Root Articles
        $articles = OOArticle::getRootArticles(false, 0);
        foreach($articles as $article) {
            $article_select->addOption($article->getName(), $article->getId());
        }

        // Root Categories
        $categories = OOCategory::getRootcategories(false, 0);

        self::getArticlesSelectChildren($article_select, $categories);

        return $article_select->get();
    }

    /**
     * HTML Select Unterelement in endloser Tiefe
     *
     * @param object $article_select	rex_select Objekt
     * @param array $categories			Kategorien
     *
     * @return string HTML-Element
     */
    public static function getArticlesSelectChildren($article_select, $categories) {

        foreach($categories as $category) {
            $tab = str_repeat('    ', (count($category->getParentTree()) - 1));
            // Kompatibilität
            if(method_exists($article_select, 'addOptgroup')) {
                $article_select->addOptgroup($tab.$category->getName());
            }
            $articles = $category->getArticles(false);
            foreach($articles as $article) {
                $article_select->addOption($tab.$article->getName(), $article->getId());
            }
            $sub_categories = $category->getChildren();
            self::getArticlesSelectChildren($article_select, $sub_categories);
        }

        return $article_select;
    }

    /**
     * Inhalte eines Slices in einen Artikel kopieren
     *
     * @param int $from_article_id 	ID des Quell-Artikels
     * @param int $to_article_id 	ID des Quell-Spalte
     * @param int $to_ctype 		ID der Ziel-Spalte
     * @param int $slice_id 		ID des Quell-Slices
     * @param int $to_clang			Sprach-ID des Ziel Artikels
     * @param int $revision			Version-ID des Artikels
     *
     * @return bool TRUE bei Erfolg, FALSE bei Fehler
     */
    static function copySlice($from_article_id, $to_article_id, $to_ctype, $slice_id, $to_clang = 0, $revision = 0) {

        global $REX, $I18N;

        $function = rex_request('function', 'string');

        // Letzte slice_id des Ziels holen
        $last_slice_in_to_article = self::getLastSliceIdForCtype($to_article_id, $to_ctype, $to_clang, $revision);

        // Quell Slice holen
        $getSliceSql = rex_sql::factory();
        // $getSliceSql->debugsql = 1;
        $getSliceSql->setQuery('
            SELECT * FROM ' . $REX['TABLE_PREFIX'] . 'article_slice
            WHERE id = ' . $slice_id . '
            AND revision = ' . $revision
        );

        if($getSliceSql->getRows() == 0) {
            // Wenn Slice nicht existiert
            throw new Exception($I18N->msg('sliceshift_msg_error_noslice'));
        }

        $addSql = rex_sql::factory();
        $addSql->setTable($REX['TABLE_PREFIX'] . 'article_slice');

        $cols = rex_sql::factory();
        // $cols->debugsql = 1;
        $cols->setquery('SHOW COLUMNS FROM ' . $REX['TABLE_PREFIX'] . 'article_slice');
        for ($j = 0; $j < $cols->rows; $j ++, $cols->next()) {
            $colname = $cols->getValue('Field');
            if ($colname == 'clang') {
                $value = $to_clang;
            } elseif ($colname == 'ctype') {
                $value = $to_ctype;
            } elseif ($colname == 're_article_slice_id') {
                $value = $last_slice_in_to_article;
            } elseif ($colname == 'article_id') {
                $value = $to_article_id;
            } elseif ($colname == 'createdate') {
                $value = time();
            } elseif ($colname == 'updatedate') {
                $value = time();
            } elseif ($colname == 'createuser') {
                $value = $REX['USER']->getValue('login');
            } elseif ($colname == 'updateuser') {
                $value = $REX['USER']->getValue('login');
            } else {
                $value = $getSliceSql->getValue($colname);
            }

            if ($colname != 'id') {
                $addSql->setValue($colname, $addSql->escape($value));
            }
        }

        // In DB einfügen
        $addSql->insert();

        // Id des eingefügten Slices (autoincrement)
        $add_slice_id = $addSql->getLastId();

        // Nachfolgenden Slice anpassen
        $updateSql = rex_sql::factory();
        // $updateSql->debugsql = 1;
        if($updateSql->setQuery('
            UPDATE ' . $REX['TABLE_PREFIX'] . 'article_slice
            SET re_article_slice_id = ' . $add_slice_id . '
            WHERE re_article_slice_id = ' . $last_slice_in_to_article . '
                AND id <> ' . $add_slice_id . '
                AND article_id = ' . $to_article_id . '
                AND clang = ' . $to_clang . '
                AND revision = ' . $revision
            )
        ) {

            $to_article = OOArticle::getArticleById($to_article_id, $to_clang);

            $article_edit_link = '<a href="index.php?page=content';
            $article_edit_link .= '&amp;mode=edit&amp;article_id=' . $to_article_id . '&amp;clang=' . $to_clang. '&amp;ctype=' . $to_ctype;
            
            // Wenn das Addon "version" installiert ist, dann version ID in URL übergeben
            if($REX['ADDON']['status']['version'] == 1) {
                $article_edit_link .= '&amp;rex_set_version=' . $revision;
            }

            $article_edit_link .= '">';
            $article_edit_link .= '"' . $to_article->getName() . ' (' . $REX['CTYPE'][$to_ctype] . ' / ' . $REX['CLANG'][$to_clang] . ')"';
            $article_edit_link .= '</a>';

            if($function == 'sliceshift_move') {
                $info = $I18N->msg('sliceshift_msg_slicemoved', $article_edit_link);
            } else {
                $info = $I18N->msg('sliceshift_msg_slicecopied', $article_edit_link);
            }

            echo rex_info($info);

        } else {
            // Wenn Fehler, dann ausgeben
            throw new Exception($I18N->msg('sliceshift_msg_error') . ' ' . $updateSql->getError());
        }

        return true;
    }

    /**
     * Letzen Slice eines Artikels und Spalte finden
     *
     * @param int $article_id 	ID des Artikels
     * @param int $ctype 		Spalten-ID des Slices
     * @param int $clang 		Sprach-ID des Slices
     * @param int $revision 	Revision-ID des Artikels
     *
     * @return int ID des Slices
     */
    public static function getLastSliceIdForCtype($article_id, $ctype, $clang = 0, $revision = 0) {

        global $REX;

        $getLastSliceSql = rex_sql::factory();
        // $getLastSliceSql->debugsql = 1;
        $getLastSliceSql->setQuery('
            SELECT a.id, a.re_article_slice_id
            FROM ' . $REX['TABLE_PREFIX'] . 'article_slice AS a
            LEFT JOIN ' . $REX['TABLE_PREFIX'] . 'article_slice AS b ON (a.id = b.re_article_slice_id AND a.ctype = b.ctype)
            WHERE a.article_id = ' .$article_id . '
            AND a.clang = ' . $clang . '
            AND a.ctype = ' . $ctype . '
            AND a.revision = ' . $revision . '
            AND b.id is NULL
        ');

        // Slice-ID zurückgeben
        if($getLastSliceSql->getRows() == 1) {
            return $getLastSliceSql->getValue('a.id');
        }

        // Wenn kein Slice vorhanden ist, dann ist die Slice-ID = 0
        return 0;
    }

    /**
     * Überprüft, ob eine Spalte in einem Template existiert
     *
     * @param array $template_attributes    Template Attribute
     * @param int $ctype_id                 Spalten-ID des Slices
     *
     * @return Boolean
     */
    public static function hasCtype($template_attributes, $ctype_id) {

        $template_ctypes = rex_getAttributes('ctype', $template_attributes, array());

        if(empty($template_ctypes)) {
            $template_ctypes = array(1 => 'default');
        }

        if(isset($template_ctypes[$ctype_id])) {
            return true;
        }

        return false;
    }
}
